@@ -251,7 +251,7 @@ class TTreeMethods(object):
                     members = streamerinfosmap[typename].members
 
         if members is not None:
-            for subbranch in branch._fBranches:
+            for subbranch in branch.itervalues(recursive=True):
                 name = subbranch._fName
                 if name.startswith(branch._fName + b"."):           # drop parent branch's name
                     name = name[len(branch._fName) + 1:]
@@ -1140,7 +1140,7 @@ class TBranchMethods(object):
             self._tryrecover()
         return self._threadsafe_key(i, keycache, False)._fObjlen
 
-    def basket_compressedbytes(self, i):
+    def basket_compressedbytes(self, i, keycache=None):
         if self._recoveredbaskets is None:
             self._tryrecover()
         key = self._threadsafe_key(i, keycache, False)
@@ -1194,7 +1194,7 @@ class TBranchMethods(object):
             byteoffsets[-1] = key._fLast
             awkward.numpy.subtract(byteoffsets, key._fKeylen, byteoffsets)
 
-        return interpretation.fromroot(data, byteoffsets, local_entrystart, local_entrystop)
+        return interpretation.fromroot(data, byteoffsets, local_entrystart, local_entrystop, key._fKeylen)
 
     def basket(self, i, interpretation=None, entrystart=None, entrystop=None, flatten=False, awkwardlib=None, cache=None, basketcache=None, keycache=None):
         awkward = _normalize_awkwardlib(awkwardlib)
@@ -2023,7 +2023,7 @@ def daskframe(path, treepath, branches=None, namedecode="utf-8", entrysteps=floa
 
 ################################################################ for quickly getting numentries
 
-def numentries(path, treepath, total=True, localsource=MemmapSource.defaults, xrootdsource=XRootDSource.defaults, httpsource=HTTPSource.defaults, executor=None, blocking=True, **options):
+def numentries(path, treepath, total=True, localsource=MemmapSource.defaults, xrootdsource={"timeout": None, "chunkbytes": 32*1024, "limitbytes": 1024**2, "parallel": False}, httpsource={"chunkbytes": 32*1024, "limitbytes": 1024**2, "parallel": False}, executor=None, blocking=True, **options):
     if isinstance(path, string_types):
         paths = _filename_explode(path)
     else:
